-- HugeBagWindow.lua
-- Originally TheOneBagWindow.lua written by Rod, Almagnus1, extended by Habna

-- HugeBag Known bugs:
-- Unload command is not working ( Known API Issues  )
-- Reload command is not working ( Known API Issues  )
-- When right clicking on a tool the icon is not swaping properly ( Known API Issues  )

-- HugeBag future functions:
-- Sorting option

-- HugeBag change log:
-- 1.0.5 (2011/05/21) : Changed the content menu into a window.
--					  : Changed the layout of the "Shell Command" window.

-- 1.0.4 (2011/05/20) : Added the option to show free or used slots informations in the title.
--					  : Added the 'About' window.
--					  : Added the 'Shell Command' window.

-- 1.0.3 (2011/05/19) : Bug fix: HugeBag is independant of any plugins manager. User can load/unload HugeBag manually with shell command.
--					  : Added the option to lock/unlock Hugebag size.
--					  : Added the option to lock/unlock Hugebag position.
--					  : Bug fix: HugeBag will not go beyond the screen border in window-mode.
--					  : Added the option for the settings to uptaded himself if a different version is detected, old user settings will remains unchanged.
--					  : Added the option to only reset location and size settings to default.
--					  : Added the option to resize HugeBag from both bottom corner.
--					  : Watermark label was removed, it will be in the 'about' window when it will be available.
--					  : Alot of functions are now in a seperate file. To clean the main file a bit.
--					  : All settings functions are now in a seperate file. To clean the main file a bit. 

-- 1.0.2 (2011/05/16) : Bug fix: Was unable to resize HugeBag when the 'reset' was launch.
--					  : Added a Watermark label of the authors Rod, Almagnus1 & Habna (me).
--					  : Added the option to toggle the hide HugeBag event when 'esc' key is press.
--					  : Change to Tobii source to be independant of Turbine plugins.

-- 1.0.1 (2011/05/15) : Added a context menu with right mouse click on the upper left corner.

-- 1.0.0 (2011/05/15) : Initial release.

--v IMPORT SECTION v--
--import "Turbine";
import "Turbine.Gameplay";
--import "Turbine.UI";
--import "Turbine.UI.Lotro";
import "HabnaPlugins.HugeBag.frmSettings"; -- LUA settings file
import "HabnaPlugins.HugeBag.functions"; -- LUA functions file

--^ IMPORT SECTION ^--
--v GLOBAL VARIABLE SECTION v--
write = Turbine.Shell.WriteLine;

frmMain = class( Turbine.UI.Lotro.Window );
--frmMain = class( Turbine.UI.Lotro.GoldWindow ); --Gold Window
--^ GLOBAL VARIABLE SECTION ^--

function frmMain:Constructor()
	--v SETTINGS SECTION v--
	Turbine.UI.Lotro.Window.Constructor( self );
	--Turbine.UI.Lotro.GoldWindow.Constructor( self ); --Gold Window

	LoadSettings();
	
	--**v Set size, backcolor, position, opacity, allowdrop and wantskeyevent of HugeBag window v**
	self:SetVisible( StartVisible );
	self:SetSize( WinWidth, WinHeight );
	self:SetBackColor( Turbine.UI.Color() );
	self:SetPosition( WinLocX, WinLocY );
	self:SetOpacity( DefaultOpacity );
	self:SetAllowDrop( true );
	self:SetWantsKeyEvents( true );
	--**^
	--**v Lock Label v**
	LockLabel = Turbine.UI.Label();
	LockLabel:SetParent( self );
	LockLabel:SetPosition( 22, 0 );
	LockLabel:SetSize(WinWidth - 50, 33 );
	if ( LockPosition ) then
		LockLabel:SetVisible( true );
	else
		LockLabel:SetVisible( false );
	end
	--**^
	-- **v Set the scrollbar v**
	self.itemListBoxScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.itemListBoxScrollBar:SetOrientation( Turbine.UI.Orientation.Vertical );
	self.itemListBoxScrollBar:SetParent( self );
	-- **^
	-- **v Set the itemlistbox v**
	self.itemListBox = Turbine.UI.ListBox();
	self.itemListBox:SetParent( self );
	self.itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal );
	self.itemListBox:SetVerticalScrollBar( self.itemListBoxScrollBar );
	-- **^	
	-- **v Get player gameplay v**
	if pcall( Turbine.Gameplay.LocalPlayer.GetInstance ) then
		self.backpack = Turbine.Gameplay.LocalPlayer.GetInstance():GetBackpack();
	else
		self.backpack = Turbine.Gameplay.LocalPlayer():GetBackpack();
	end
	-- **^
	self:SetBackPack(); --> ** Set the backpack **
	--^ SETTINGS SECTION ^--
	--v EVENT HANDLERS SECTION v--
	--**v HugeBag event handlers v**
	-- **v Set the opacity when the mouse enter the HugeBag v**
	self.MouseEnter = function( sender, args )
		sender:SetOpacity( 1.0 );
	end
	-- **^
	-- **v Set the opacity when the mouse leave the HugeBag v**
	self.MouseLeave = function( sender, args )
		sender:SetOpacity( DefaultOpacity );
	end
	-- **^
	-- **v MouseDonw event on HugeBag v**
	self.MouseDown = function( sender, args )
		if ( args.Button == Turbine.UI.MouseButton.Left ) then
			if ( args.X < 30 and args.Y < 50 )then
				CornerClick = "TopLeft";
				--self.menu:ShowMenu()
				frmSettings()
				--HelpInfo()
			elseif ( ( args.X < 30 ) and ( args.Y > ( self:GetHeight() - 30 ) ) ) then
				CornerClick = "BottomLeft";
				dragStartX = args.X;
				dragStartY = args.Y;
				dragging = true;
			elseif ( args.X > ( self:GetWidth() -30 ) ) and ( args.Y > ( self:GetHeight() - 30 ) ) then
				CornerClick = "BottomRight";
				dragStartX = args.X;
				dragStartY = args.Y;
				dragging = true;
			end
		end
		-- ** To prevent from lock the context menu, le lock must nullify the dragging
		if ( LockSize ) then
			dragging = false;
		end
	end
	-- **^
	-- **v MouseUp event on HugeBag v**
	self.MouseUp = function( sender, args )
		dragging = false;
		OutOfBound = false;
		CornerClick = "";
		SaveSettings()
	end
	-- **^
	-- **v MouseMouve event on HugeBag v**
	self.MouseMove = function( sender, args )
		if ( dragging ) then
			WinWidth, WinHeight = self:GetSize();
			WinLocX, WinLocY = self:GetPosition();
			local screenWidth = Turbine.UI.Display:GetWidth();
			local screenHeight = Turbine.UI.Display:GetHeight();
			local tempY = WinHeight + ( args.Y - dragStartY );

			if CornerClick == "BottomRight" then
				-- **v Check horizontal v**
				local tempX = WinWidth + ( args.X - dragStartX );
				if ( WinLocX + tempX ) > screenWidth then
					self:SetSize( ( screenWidth - WinLocX ), WinHeight );
				else
					self:SetSize( tempX, tempY );
					dragStartX = dragStartX + (args.X - dragStartX);
					dragStartY = dragStartY + (args.Y - dragStartY);
				end
				-- **^
			elseif CornerClick == "BottomLeft" then
				-- **v Check horizontal v**
				local tempX = WinWidth - ( args.X - dragStartX );
				if WinLocX < 0 then
					dragging = false;
					self:SetPosition( 0, WinLocY );
					--self:SetSize( WinWidth , WinHeight );
				else
					self:SetPosition( WinLocX + ( args.X - dragStartX ), WinLocY );
					self:SetSize( tempX, tempY );
					dragStartY = dragStartY + ( args.Y - dragStartY );
				end
				-- **^
			end
			-- **v Check vertical v**
			if ( WinLocY + tempY ) > screenHeight then
				self:SetSize( WinWidth, ( screenHeight - WinLocY ) );
			end
			-- **^
			self:PerformLayout()
		end
	end
	-- **^
	-- **v Do stuff before closing the HugeBag v**
	self.Closing = function( sender, args )
		
	end
	-- **^
	-- **v Refresh and save the X,Y positiov when HugeBag position changed v**
	self.PositionChanged = function(args)
		WinLocX, WinLocY = self:GetPosition()
		SaveSettings()
	end
	-- **^
	-- **v Detect key press v**
	self.KeyDown = function( sender, args )
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			if ( HideWEsc ) then
				self:SetVisible( false ); -- **v Hide HugeBag if ESC key is press v**
			end
		end
		
		if ( args.Action == Turbine.UI.Lotro.Action.ToggleBags or
		     args.Action == Turbine.UI.Lotro.Action.ToggleBag1 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag2 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag3 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag4 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag5 )
		then
			sender:SetVisible( not sender:IsVisible() ) -- **v Show/Hide HugeBag if any bag was toggle v**
		end
	end
	-- **^
	-- **v HugeBag siez as changed v**
	self.backpack.SizeChanged = function( sender, args )
		self:SetBackPack();
	end
	-- **^
	-- **v Adding an item to HugeBag v**
	self.backpack.ItemAdded = function( sender, args )
		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		freeslots = freeslots -1;
		self:PrintTitle()
	end
	-- **^
	-- **v Removing an item to HugeBag v**
	self.backpack.ItemRemoved = function( sender, args )
		self.items[args.Index]:SetItem( self.backpack:GetItem( args.Index ) );
		freeslots = freeslots +1;
		self:PrintTitle()
	end
	-- **^
	-- **v Swap item in HugeBag when moved v**
	self.backpack.ItemMoved = function( sender, args )
		--if sender is the bag then it's from within the bag so swap both index
		self.items[args.OldIndex]:SetItem( self.backpack:GetItem( args.OldIndex ) );
		self.items[args.NewIndex]:SetItem( self.backpack:GetItem( args.NewIndex ) );
		--else
			-- the sender is not the bag so swap icon at the same index (ex.: when using a tool)

		--end
	end

	self.DragDrop = function( sender, args )
		local shortcut = args.DragDropInfo:GetShortcut();
		local destinationItemControl = self.itemListBox:GetItemAt( args.X - InitXPos, args.Y - InitYPos );
		
		if ( destinationItemControl ~= nil ) then -- ** if nil, then it's out of range, so don't do anything **
			local destinationIndex = self.itemListBox:IndexOfItem( destinationItemControl );
			self.backpack:PerformShortcutDrop(shortcut, destinationIndex, Turbine.UI.Control.IsShiftKeyDown() );
		end
		
	end
	-- **^
	--**^ HugeBag event handlers ^**
	--^ EVENT HANDLERS SECTION ^--
end
-- **^
--v FUNCTIONS/METHODS SECTION v--
-- **v Refresh the backpack list v**
function frmMain:SetBackPack()
	self.items = nil;
	self.itemListBox:ClearItems();

	self.items = {}; --> ** Initialize the container that holds the elements referencing the player's inventory **
	backpackSize = self.backpack:GetSize();

	local temp;
	local tName;

	for i = 1, backpackSize, 1 do
		if ( self.items[i] ) then
			self.items[i]:SetParent( nil );
		end
		
		-- Get item and set background color
		--self.items[i] = Turbine.UI.Lotro.ItemControl( self.backpack:GetItem( i ) );
		temp = Turbine.UI.Lotro.ItemControl( self.backpack:GetItem( i ) );
		temp:SetBackground( "HabnaPlugins/HugeBag/Resources/slotBackground.tga" );
		temp:SetBlendMode( Turbine.UI.BlendMode.Overlay );
		self.items[i] = temp;
		self.items[i].id = i; -- For sorting purpose

		self.itemListBox:AddItem( self.items[i] );
	end
	
	self:PerformLayout();
	self:CountFreeSlots()
end
-- **^
-- **v Layout of the items in the listbox v**
function frmMain:PerformLayout()
	WinWidth, WinHeight = self:GetSize();
	WinLocX, WinLocY = self:GetPosition()
	local itemWidth = 40;

	if ( self.items[1] ~= nil ) then
		itemWidth = self.items[1]:GetWidth()
	end

	local listWidth = WinWidth - 40;
	local listHeight = WinHeight - 50;
	local itemsPerRow = listWidth / itemWidth;
	
	self.itemListBox:SetPosition( InitXPos, InitYPos );
	self.itemListBox:SetSize( listWidth, listHeight );
	self.itemListBox:SetMaxItemsPerLine( itemsPerRow );
	
	self.itemListBoxScrollBar:SetPosition( WinWidth - 17, 40 );
	self.itemListBoxScrollBar:SetSize( 10, listHeight - 15 );
	
	LockLabel:SetSize(WinWidth - 50, 33 ); --> ** Resize width the LockLabel with HugeBag width **
end
-- **^
-- **v Count the free slots v**
function frmMain:CountFreeSlots()
	freeslots = 0;

	for i = 1, backpackSize, 1 do
		if ( self.backpack:GetItem( i ) == nil ) then
			freeslots = freeslots + 1;
		end
	end

	self:PrintTitle()
end
-- **^
-- **v Print the title with the free slots and the max v**
function frmMain:PrintTitle()
	if ( ShowSlotsInfo ) then
		if ( ToggleUsed ) then 
			local usedslots = backpackSize - freeslots;
			self:SetText( "HugeBag - used slots: " .. usedslots .. " / " .. backpackSize );
		else
			self:SetText( "HugeBag - free slots: " .. freeslots .. " / " .. backpackSize );
		end
	else
		self:SetText( "HugeBag" );
	end
end
-- **^
--^ FUNCTIONS/METHODS SECTION ^--